@@ -1,5 +1,6 @@
 from functools import partial, wraps
 
+from pipetools.debug import repr_args, set_name, get_name
 from pipetools.ds_builder import DSBuilder, NoBuilder
 from pipetools.main import pipe, XObject, StringFormatter
 
@@ -8,15 +9,19 @@ def pipe_util(func):
     """
     Decorator that handles X objects and currying for pipe-utils.
     """
-    @wraps(func)
     def pipe_util_wrapper(function, *args, **kwargs):
         if isinstance(function, XObject):
             function = ~function
 
+        function_name = get_name(function)
+
         if args or kwargs:
             function = partial(function, *args, **kwargs)
 
-        return pipe | func(function)
+        name = '%s(%s)' % (func.__name__, ', '.join(
+            filter(None, (function_name, repr_args(*args, **kwargs)))))
+
+        return pipe | set_name(name, func(function))
 
     return pipe_util_wrapper
 
