@@ -1,5 +1,8 @@
 from functools import partial
 
+from pipetools.debug import get_name, set_name, repr_args
+from pipetools.debug import pipe_exception_handler
+
 
 class Pipe(object):
     """
@@ -14,10 +17,19 @@ class Pipe(object):
     """
     def __init__(self, func=None):
         self.func = func
+        self.__name__ = str(self)
+
+    def __str__(self):
+        return get_name(self.func)
 
     @staticmethod
     def compose(first, second):
-        return lambda *args, **kwargs: second(first(*args, **kwargs))
+        name = '{0} | {1}'.format(get_name(first), get_name(second))
+
+        def composite(*args, **kwargs):
+            with pipe_exception_handler('pipe | ' + name):
+                return second(first(*args, **kwargs))
+        return set_name(name, composite)
 
     @classmethod
     def bind(cls, first, second):
@@ -45,10 +57,13 @@ class Maybe(Pipe):
 
     @staticmethod
     def compose(first, second):
+        name = '{0} ?| {1}'.format(get_name(first), get_name(second))
+
         def composite(*args, **kwargs):
-            result = first(*args, **kwargs)
-            return None if result is None else second(result)
-        return composite
+            with pipe_exception_handler('maybe | ' + name):
+                result = first(*args, **kwargs)
+                return None if result is None else second(result)
+        return set_name(name, composite)
 
     def __lt__(self, thing):
         return (
@@ -82,7 +97,7 @@ def StringFormatter(template):
             return f(*content)
         return f(content)
 
-    return format
+    return set_name("format('%s')" % template[:20], format)
 
 
 def _iterable(obj):
@@ -94,49 +109,61 @@ def _iterable(obj):
 class XObject(object):
 
     def __init__(self, func=None):
-        self._func = func or (lambda x: x)
+        self._func = func
+        self.__name__ = get_name(func)
+
+    def __repr__(self):
+        return self.__name__ or 'X'
 
     def __invert__(self):
-        return self._func
+        return self._func or set_name('X', lambda x: x)
+
+    def bind(self, name, func):
+        try:
+            func.__name__ = str(name)
+        except UnicodeError:
+            func.__name__ = repr(name)
+        return XObject((self._func | func) if self._func else (pipe | func))
 
     def __call__(self, *args, **kwargs):
-        return XObject(pipe | self | (lambda x: x(*args, **kwargs)))
+        name = 'X(%s)' % repr_args(*args, **kwargs)
+        return self.bind(name, lambda x: x(*args, **kwargs))
 
     def __eq__(self, other):
-        return XObject(pipe | self | (lambda x: x == other))
+        return self.bind('X == %s' % other, lambda x: x == other)
 
     def __getattr__(self, name):
-        return XObject(pipe | self | (lambda x: getattr(x, name)))
+        return self.bind('X.%s' % name, lambda x: getattr(x, name))
 
     def __getitem__(self, item):
-        return XObject(pipe | self | (lambda x: x[item]))
+        return self.bind('X[%s]' % item, lambda x: x[item])
 
     def __gt__(self, other):
-        return XObject(pipe | self | (lambda x: x > other))
+        return self.bind('X > %s' % other, lambda x: x > other)
 
     def __lt__(self, other):
-        return XObject(pipe | self | (lambda x: x < other))
+        return self.bind('X < %s' % other, lambda x: x < other)
 
     def __mod__(self, y):
-        return XObject(pipe | self | (lambda x: x % y))
+        return self.bind('X %% %s' % y, lambda x: x % y)
 
     def __ne__(self, other):
-        return XObject(pipe | self | (lambda x: x != other))
+        return self.bind('X != %s' % other, lambda x: x != other)
 
     def __neg__(self):
-        return XObject(pipe | self | (lambda x: -x))
+        return self.bind('-X', lambda x: -x)
 
     def __mul__(self, other):
-        return XObject(pipe | self | (lambda x: x * other))
+        return self.bind('X * %s' % other, lambda x: x * other)
 
     def __add__(self, other):
-        return XObject(pipe | self | (lambda x: x + other))
+        return self.bind('X + %s' % other, lambda x: x + other)
 
     def __ror__(self, func):
         return pipe | func | self
 
     def _in_(self, container):
-        return XObject(pipe | self | (lambda x: x in container))
+        return self.bind('X._in_(%s)' % container, lambda x: x in container)
 
 
 X = XObject()
